@@ -1,6 +1,6 @@
 #
-# Copyright (C) 2001-2005 Ichiro Fujinaga, Michael Droettboom,
-#                          and Karl MacMillan
+# Copyright (C) 2001-2005 Ichiro Fujinaga, Michael Droettboom, Karl MacMillan
+#               2009-2010 Christoph Dalitz
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -719,18 +719,22 @@ connnected components, such as the lower-case *i*.
 .. note::
    Here *id* is a simple string, not of the `id_name`_ format, since
    the confidence of a manual classification is always 1.0."""
-      if id.startswith('_group') and len(glyphs) > 1:
-         import image_utilities
-         parts = id.split('.')
-         sub = '.'.join(parts[1:])
-         union = image_utilities.union_images(glyphs)
-         for glyph in glyphs:
-            if glyph.nrows > 2 and glyph.ncols > 2:
-               glyph.classify_heuristic('_group._part.' + sub)
-               self.generate_features(glyph)
-         added, removed = self.classify_glyph_manual(union, sub)
-         added.append(union)
-         return added, removed
+      if id.startswith('_group'):
+         if len(glyphs) > 1:
+            import image_utilities
+            parts = id.split('.')
+            sub = '.'.join(parts[1:])
+            union = image_utilities.union_images(glyphs)
+            for glyph in glyphs:
+               if glyph.nrows > 2 and glyph.ncols > 2:
+                  glyph.classify_heuristic('_group._part.' + sub)
+                  self.generate_features(glyph)
+            added, removed = self.classify_glyph_manual(union, sub)
+            added.append(union)
+            return added, removed
+         else:
+            # grouping a single glyph corrupts the classifier_glyph.xml file
+            raise ClassifierError("Grouping of only a single glyph is not allowed.")
 
       added = []
       removed = util.sets.Set()
