@@ -1,8 +1,8 @@
 # -*- mode: python; indent-tabs-mode: nil; tab-width: 3 -*-
 # vim: set tabstop=3 shiftwidth=3 expandtab:
 #
-# Copyright (C) 2001-2005 Ichiro Fujinaga, Michael Droettboom,
-#                          and Karl MacMillan
+# Copyright (C) 2001-2005 Ichiro Fujinaga, Michael Droettboom, Karl MacMillan
+#               2009-2010 Christoph Dalitz
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -32,6 +32,7 @@ Image - images.
 SubImage - a view on part of an image.
 Cc - a non-rectangular view of part of an image (the view area
      is marked with labels in the image).
+MlCc - the same as Cc but with multiple labels
 
 Additionally this module contains the following functions:
 
@@ -261,7 +262,7 @@ the same data."""
    def set_display(self, _display):
       self._display = _display
 
-   def display(self):
+   def display(self, title=None):
       """Displays the image in its own window.  (See `Using the Gamera GUI`__).  If the GUI
 process is not running, this method has no effect.
 
@@ -275,9 +276,14 @@ process is not running, this method has no effect.
       if self._display:
          self._display.set_image(self, conversion)
       else:
-         self.set_display(
-            has_gui.gui.ShowImage(self, self.name, conversion,
-                          owner=self))
+         if title == None:
+            self.set_display(
+               has_gui.gui.ShowImage(self, self.name, conversion,
+                                     owner=self))
+         else:
+            self.set_display(
+               has_gui.gui.ShowImage(self, title, conversion,
+                                     owner=self))
       self.last_display = "normal"
       return self._display
 
@@ -357,7 +363,7 @@ This method also unsets the *confidence* map.
    def classify_automatic(self, id_name):
       """Classifies the image as the value *id_name* and sets the state
 to AUTOMATIC.  Use this method when an automatic classifier has classified
-this glyph. Note that this method does not the *confidence* map for
+this glyph. Note that this method does not touch the *confidence* map for
 the main id; this must be set separately.
 
 *id_name*
@@ -382,7 +388,7 @@ the main id; this must be set separately.
 
    def classify_heuristic(self, id_name):
       """Classifies the image as the value *id_name* and sets the state
-to AUTOMATIC.  Use this method when a heuristic process has classified
+to HEURISTIC.  Use this method when a heuristic process has classified
 this glyph. This method also unsets the *confidence* map.
 
 *id_name*
@@ -428,6 +434,15 @@ If the image is unclassified, the result is 'UNCLASSIFIED'.
 
 When no *confidence_type* is given, the default confidence stored with
 ``id_name[0]`` is returned. If the image is unclassified, -1.0 is returned.
+
+For details about the confidence computation in Gamera, see
+
+   C. Dalitz: `Reject Options and Confidence Measures for kNN Classifiers.`__
+   In C. Dalitz (Ed.): "Document Image Analysis with the Gamera Framework."
+   Schriftenreihe des Fachbereichs Elektrotechnik und Informatik,
+   Hochschule Niederrhein, vol. 8, pp. 16-38, Shaker Verlag (2009).
+
+.. __: http://lionel.kr.hsnr.de/~dalitz/data/publications/sr09-knn-rejection.pdf
 """
       if self.classification_state == UNCLASSIFIED:
          return -1.0
@@ -462,6 +477,8 @@ There are a number of ways to create a subimage:
 Changes to subimages will affect all other subimages viewing the same data."""
       if hasattr(self, "label"):
          return Cc(self, self.label, *args, **kwargs)
+      if self.__class__.__name__=="MlCc": #seems more robust to me
+         return self.copy(*args, **kwargs)
       else:
          return SubImage(self, *args, **kwargs)
 
@@ -601,6 +618,16 @@ is not running.
       self._display.focus(self)
       self.last_display = "context"
 
+class MlCc(gameracore.MlCc, ImageBase):
+   def __init__(self, *args, **kwargs):
+      ImageBase.__init__(self)
+      gameracore.MlCc.__init__(self, *args, **kwargs)
+   __init__.__doc__ = gameracore.MlCc.__doc__
+
+   def __del__(self):
+      if self._display:
+         self._display.close()
+
 # this is a convenience function for using in a console
 _gamera_initialised = False
 def _init_gamera():
@@ -698,6 +725,6 @@ __all__ = ("init_gamera UNCLASSIFIED AUTOMATIC HEURISTIC MANUAL "
            "CONFIDENCE_LINEARWEIGHT CONFIDENCE_INVERSEWEIGHT "
            "CONFIDENCE_NUN CONFIDENCE_NNDISTANCE CONFIDENCE_AVGDISTANCE "
            "ImageData Size Dim Point FloatPoint Rect Region RegionMap "
-           "ImageInfo Image SubImage Cc load_image image_info "
+           "ImageInfo Image SubImage Cc MlCc load_image image_info "
            "display_multi ImageBase nested_list_to_image RGBPixel "
            "save_image").split()
