@@ -27,6 +27,7 @@ import string
 from gamera import util, enums
 from gamera.gui import gui_util
 from gamera.core import RGBPixel
+from gamera.args import DEFAULT_MAX_ARG_NUMBER, CNoneDefault
 import sys
 
 class ArgInvalidException(Exception):
@@ -364,6 +365,8 @@ class Class:
             for key, val in locals.items():
                if isinstance(val, self.klass):
                   choices.append(key)
+      if isinstance(self.default, CNoneDefault):
+         choices = ["None"] + choices
       return choices
 
    def get_control(self, parent, locals=None):
@@ -371,7 +374,7 @@ class Class:
          self.klass = eval(self.klass)
       self.control = wx.Choice(
          parent, -1, choices = self.determine_choices(locals))
-      self.control.SetSelection(self.default)
+      #self.control.SetSelection(0)
       return self
 
    def get(self):
@@ -751,14 +754,14 @@ class Point:
       self.control.Add(wx.StaticText(parent, -1, "x:"))
       self.control_x = wx.SpinCtrl(
          parent, -1, value=str(default.x),
-         min=-sys.maxint, max=sys.maxint,
+         min=-DEFAULT_MAX_ARG_NUMBER, max=DEFAULT_MAX_ARG_NUMBER,
          initial=default.x)
       self.control_x.SetValidator(_IntValidator(name=self.name))
       self.control.Add(self.control_x, 1, wx.EXPAND | wx.LEFT | wx.RIGHT, 5)
       self.control.Add(wx.StaticText(parent, -1, "y:"))
       self.control_y = wx.SpinCtrl(
          parent, -1, value=str(default.y),
-         min=-sys.maxint, max=sys.maxint,
+         min=-DEFAULT_MAX_ARG_NUMBER, max=DEFAULT_MAX_ARG_NUMBER,
          initial=default.y)
       self.control_y.SetValidator(_IntValidator(name=self.name))
       self.control.Add(self.control_y, 1, wx.EXPAND | wx.LEFT, 5)
