@@ -287,11 +287,11 @@ class PageMultiImageDisplay(ExtendedMultiImageDisplay):
          self.SelectBlock(row, col, row, col, 0)
          self.MakeCellVisible(min(row + 1, self.rows), col)
          if image.classification_state != MANUAL:
-            id = self.toplevel.guess_glyph(image)
+            id = self.toplevel.guess_glyph(image)[0][0][1]
          else:
-            id = image.id_name
-         self.toplevel.set_label_display(id[0][1])
-         self.toplevel.display_label_at_cell(row, col, id[0][1])
+            id = image.id_name[0][1]
+         self.toplevel.set_label_display(id)
+         self.toplevel.display_label_at_cell(row, col, id)
 
    def move_back(self):
       row = self.GetGridCursorRow()
@@ -305,11 +305,11 @@ class PageMultiImageDisplay(ExtendedMultiImageDisplay):
       self.MakeCellVisible(min(row + 1, self.rows), col)
       image = self.sorted_glyphs[image_no]
       if image.classification_state != MANUAL:
-         id = self.toplevel.guess_glyph(image)
+         id = self.toplevel.guess_glyph(image)[0][0][1]
       else:
-         id = image.id_name
-      self.toplevel.set_label_display(id[0][1])
-      self.toplevel.display_label_at_cell(row, col, id[0][1])
+         id = image.id_name[0][1]
+      self.toplevel.set_label_display(id)
+      self.toplevel.display_label_at_cell(row, col, id)
 
    ########################################
    # DISPLAYING A LABEL BENEATH A CELL
@@ -625,6 +625,7 @@ class ClassifierFrame(ImageFrameBase):
          self._frame,
          (("&Open and segment image...", self._OnOpenAndSegmentImage),
           ("Se&lect and segment image...", self._OnSelectAndSegmentImage),
+          ("Select already &segmented image...", self._OnSelectSegmentedImage),
           ("Se&lect image...", self._OnSelectImage),
           (None, None),
           ("&Save glyphs into separate files",
@@ -836,7 +837,13 @@ class ClassifierFrame(ImageFrameBase):
          try:
             added, removed = self._classifier.classify_list_manual(selection, id)
          except ClassifierError, e:
+            cursorbusy = False
+            if wx.IsBusy():
+               cursorbusy = True
+               wx.EndBusyCursor()
             gui_util.message(str(e))
+            if cursorbusy:
+               wx.BeginBusyCursor()
             return
          if len(added) or len(removed):
             wx.BeginBusyCursor()
@@ -1392,6 +1399,22 @@ class ClassifierFrame(ImageFrameBase):
          return
       wx.EndBusyCursor()
 
+   def _OnSelectSegmentedImage(self, event):
+      if self.multi_iw.id.is_dirty:
+         if not gui_util.are_you_sure_dialog("Page glyphs have not been saved.  Are you sure you wish to proceed?"):
+            return
+      dialog = Args(
+         [Class("Image", ImageBase),
+          Class("Ccs", ImageBase, list_of=True)],
+         name="Select image and ccs...")
+      results = dialog.show(
+         self._frame, image_menu.shell.locals,
+         docstring = "Select an image and its segmented glyphs")
+      if results is None:
+         return
+      (image, ccs) = results
+      self.set_image(ccs, image, weak=False)
+
    def _OnSelectImage(self, event):
       dialog = Args(
          [Class("Image", ImageBase)],
