@@ -1,7 +1,7 @@
 /*
  *
- * Copyright (C) 2001-2005
- * Ichiro Fujinaga, Michael Droettboom, and Karl MacMillan
+ * Copyright (C) 2001-2005 Ichiro Fujinaga, Michael Droettboom, Karl MacMillan
+ *               2010      Christoph Dalitz
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -22,6 +22,7 @@
 #include "vigra/resizeimage.hxx"
 #include "vigra/affinegeometry.hxx"
 #include "plugins/logical.hpp"
+#include "plugins/morphology.hpp"
 
 #include <exception>
 #include <cstdlib>
@@ -30,120 +31,31 @@
 #include <time.h>
 #include <algorithm>
 
-/*
- * Rotate at an arbitrary angle.
- *
- * This algorithm works by first rotating for 90 degrees, depending whether
- * height and width are exchanged by rotation or not.
- * Afterwards VIGRA's rotation algorithm is called, which allows
- * for different types of interpolation.
- *
- * src - A view of of the source image
- * angle - Degree of rotation
- * bgcolor - Background color
- *
- */
-template<class T>
-typename ImageFactory<T>::view_type* rotate(const T &src, double angle, typename T::value_type bgcolor, int order)
+// for backward compatibility:
+// plugin rotate has been moved from here to transformation.hpp
+#include "transformation.hpp"
+
+namespace Gamera {
+
+inline RGBPixel norm_weight_avg(RGBPixel& pix1, RGBPixel& pix2, double w1=1.0, double w2=1.0)
 {
-  if (order < 1 || order > 3) {
-    throw std::range_error("Order must be between 1 and 3");
-  }
-  
-  // Adjust angle to a positive double between 0-360
-  while(angle<0.0) angle+=360;
-  while(angle>=360.0) angle-=360;
+  if(w1 == -w2) w1 = w2 = 1.0;
+  return RGBPixel( GreyScalePixel(((pix1.red() * w1) + (pix2.red() * w2))/(w1 + w2)),
+		   GreyScalePixel(((pix1.green() * w1) + (pix2.green() * w2))/(w1 + w2)),
+		   GreyScalePixel(((pix1.blue() * w1) + (pix2.blue() * w2))/(w1 + w2)));
+}
 
-  // some angle ranges flip width and height
-  // as VIGRA requires source and destination to be of the same
-  // size, it cannot handle a reduce in one image dimension.
-  // Hence we must rotate by 90 degrees, if necessary
-  bool rot90done = false;
-  typename ImageFactory<T>::view_type* prep4vigra = (typename ImageFactory<T>::view_type*) &src;
-  if ((45 < angle && angle < 135) ||
-      (225 < angle && angle < 315)) {
-     typename ImageFactory<T>::data_type* prep4vigra_data =
-       new typename ImageFactory<T>::data_type(Size(src.height(),src.width()));
-     prep4vigra = new typename ImageFactory<T>::view_type(*prep4vigra_data);
-     size_t ymax = src.nrows() - 1;
-     for (size_t y = 0; y < src.nrows(); ++y) {
-       for (size_t x = 0; x < src.ncols(); ++x) {
-         prep4vigra->set(Point(ymax-y,x), src.get(Point(x,y)));
-       }
-     }
-     rot90done = true;
-     // recompute rotation angle, because partial rotation already done
-     angle -= 90.0;
-     if (angle < 0.0) angle +=360;
-  }
+inline OneBitPixel norm_weight_avg(OneBitPixel& pix1, OneBitPixel& pix2, double w1=1.0, double w2=1.0)
+{
+  if(w1 == -w2) w1 = w2 = 1.0;
+  if(((pix1 * w1) + (pix2 * w2))/(w1 + w2) < 0.5) return OneBitPixel(0);
+  return OneBitPixel(1);
+}
 
-  double rad = (angle / 180.0) * M_PI;
-
-  // new width/height depending on angle
-  size_t new_width, new_height;
-  if ((0 <= angle && angle <= 90) ||
-      (180 <= angle && angle <= 270)) {
-    new_width = size_t(0.5+abs(cos(rad) * (double)prep4vigra->width() + 
-                           sin(rad) * (double)prep4vigra->height()));
-    new_height = size_t(0.5+abs(sin(rad) * (double)prep4vigra->width() + 
-                            cos(rad) * (double)prep4vigra->height()));
-  } else {
-    new_width = size_t(0.5+abs(cos(rad) * (double)prep4vigra->width() - 
-                           sin(rad) * (double)prep4vigra->height()));
-    new_height = size_t(0.5+abs(sin(rad) * (double)prep4vigra->width() - 
-                            cos(rad) * (double)prep4vigra->height()));
-  }
-  size_t pad_width = 0;
-  if (new_width > prep4vigra->width())
-    pad_width = (new_width - prep4vigra->width()) / 2 + 2;
-  size_t pad_height = 0;
-  if (new_height > prep4vigra->height())
-    pad_height = (new_height - prep4vigra->height()) / 2 + 2;
-
-  typename ImageFactory<T>::view_type* tmp =
-    pad_image(*prep4vigra, pad_height, pad_width, pad_height, pad_width, bgcolor);
-
-  typename ImageFactory<T>::data_type* dest_data =
-    new typename ImageFactory<T>::data_type(tmp->size());
-  typename ImageFactory<T>::view_type* dest =
-    new typename ImageFactory<T>::view_type(*dest_data);
-
-  try {
-    fill(*dest, bgcolor);
-
-    if (order == 1) {
-      vigra::SplineImageView<1, typename T::value_type> 
-	spline(src_image_range(*tmp));
-      vigra::rotateImage(spline, dest_image(*dest), -angle);
-    } else if (order == 2) {
-      vigra::SplineImageView<2, typename T::value_type> 
-	spline(src_image_range(*tmp));
-      vigra::rotateImage(spline, dest_image(*dest), -angle);
-    } else if (order == 3) {
-      vigra::SplineImageView<3, typename T::value_type> 
-	spline(src_image_range(*tmp));
-      vigra::rotateImage(spline, dest_image(*dest), -angle);
-    }
-  } catch (std::exception e) {
-    delete tmp->data();
-    delete tmp;
-    delete dest;
-    delete dest_data;
-    if (rot90done) {
-      delete prep4vigra->data();
-      delete prep4vigra;
-    }
-    throw;
-  }
-
-  if (rot90done) {
-    delete prep4vigra->data();
-    delete prep4vigra;
-  }
-  delete tmp->data();
-  delete tmp;
-
-  return dest;
+template <class T>
+inline T norm_weight_avg(T& pix1, T& pix2, double w1=1.0, double w2=1.0)
+{
+  return T(((pix1 * w1) + (pix2 * w2))/(w1 + w2));
 }
 
 /*
@@ -687,25 +599,180 @@ typename ImageFactory<T>::view_type* ink_diffuse(const T &src, int type, double
   return new_view;
 }
 
-inline RGBPixel norm_weight_avg(RGBPixel& pix1, RGBPixel& pix2, double w1=1.0, double w2=1.0)
+
+/*
+ * Image degradation after Kanungo et al.
+ */
+template<class T>
+typename ImageFactory<T>::view_type* degrade_kanungo(const T &src, float eta, float a0, float a, float b0, float b, int k, int random_seed = 0)
 {
-  if(w1 == -w2) w1 = w2 = 1.0;
-  return RGBPixel( GreyScalePixel(((pix1.red() * w1) + (pix2.red() * w2))/(w1 + w2)),
-		   GreyScalePixel(((pix1.green() * w1) + (pix2.green() * w2))/(w1 + w2)),
-		   GreyScalePixel(((pix1.blue() * w1) + (pix2.blue() * w2))/(w1 + w2)));
+  typedef typename ImageFactory<T>::data_type data_type;
+  typedef typename ImageFactory<T>::view_type view_type;
+  typedef typename T::value_type value_type;
+  int d;
+  double randval;
+
+  FloatImageView *dt_fore, *dt_back;
+  typename T::const_vec_iterator p;
+  typename view_type::vec_iterator q;
+  FloatImageView::vec_iterator df,db;
+  value_type blackval = black(src);
+  value_type whiteval = white(src);
+
+  data_type* dest_data = new data_type(src.size(), src.origin());
+  view_type* dest = new view_type(*dest_data);
+  
+  // compute distance transform of foreground and background
+  // as dest is not yet needed we abuse it for storing the inverted image
+  dt_fore = (FloatImageView*)distance_transform(src, 0);
+  for (p=src.vec_begin(), q=dest->vec_begin(); p != src.vec_end(); p++, q++) {
+    if (is_black(*p)) *q = whiteval;
+    else *q = blackval;
+  }
+  dt_back = (FloatImageView*)distance_transform(*dest, 0);
+
+  // precompute probabilities (maximum distance 32 should be enough)
+  double P_foreground_flip[32];
+  double P_background_flip[32];
+  for (d=0; d<32; d++) {
+    P_foreground_flip[d] = a0*exp(-a*(d+1)*(d+1)) + eta;
+    P_background_flip[d] = b0*exp(-b*(d+1)*(d+1)) + eta;
+  }
+
+  // flip pixels randomly based on their distance from border
+  srand(random_seed);
+  for (q=dest->vec_begin(), df=dt_fore->vec_begin(), db=dt_back->vec_begin();
+       q != dest->vec_end(); q++, df++, db++) {
+    randval = ((double)rand()) / RAND_MAX;
+    // note that dest is still inverted => black is background!!
+    if (is_black(*q)) {
+      d = (int)(*db + 0.5);
+      if ((d > 32) || (randval > P_background_flip[d-1]))
+        *q = whiteval;
+    } else {
+      d = (int)(*df + 0.5);
+      if ((d > 32) || (randval > P_foreground_flip[d-1]))
+        *q = blackval;
+    }
+  }
+
+  // do a morphological closing
+  if (k>1) {
+    // build structuring element
+    data_type* se_data = new data_type(Dim(k,k), Point(0,0));
+    view_type* se = new view_type(*se_data);
+    for (q=se->vec_begin(); q!=se->vec_end(); q++)
+      *q = blackval;
+    view_type* dilated = dilate_with_structure(*dest, *se, Point(k/2,k/2));
+    view_type* eroded = erode_with_structure(*dilated, *se, Point(k/2,k/2));
+    delete dilated->data(); delete dilated;
+    delete dest->data(); delete dest;
+    delete se_data; delete se;
+    dest = eroded;
+  }
+
+  // clean up
+  delete dt_fore->data(); delete dt_fore;
+  delete dt_back->data(); delete dt_back;
+
+  return dest;
 }
 
-inline OneBitPixel norm_weight_avg(OneBitPixel& pix1, OneBitPixel& pix2, double w1=1.0, double w2=1.0)
+
+/*
+ * add white speckles in onebit image
+ */
+template<class T>
+Image* white_speckles(const T &src, float p0, int n, int k, int connectivity = 2, int random_seed = 0)
 {
-  if(w1 == -w2) w1 = w2 = 1.0;
-  if(((pix1 * w1) + (pix2 * w2))/(w1 + w2) < 0.5) return OneBitPixel(0);
-  return OneBitPixel(1);
+  typedef typename ImageFactory<T>::data_type data_type;
+  typedef typename ImageFactory<T>::view_type view_type;
+  typedef typename T::value_type value_type;
+  double randval;
+  size_t x,y;
+  size_t maxx = src.ncols() - 1;
+  size_t maxy = src.nrows() - 1;
+  int i;
+
+  value_type blackval = black(src);
+  value_type whiteval = white(src);
+
+  data_type* speckles_data = new data_type(src.size(), src.origin());
+  view_type* speckles = new view_type(*speckles_data);
+
+  // create random walk data
+  for (y=0; y <= maxy; y++) {
+    for (x=0; x <= maxx; x++) {
+      Point p(x,y);
+      if (is_black(src.get(p)) && (((double)rand()) / RAND_MAX < p0)) {
+        speckles->set(p,blackval);
+        for (i=0; i<n; i++) {
+          if (p.x() == 0 || p.x() == maxx || p.y() == 0 || p.y() == maxy)
+            break;
+          randval = ((double)rand()) / RAND_MAX;
+          if (connectivity == 0) {
+            // random rook move
+            if (randval < 0.25)      p.x(p.x() + 1);
+            else if (randval < 0.5)  p.x(p.x() - 1);
+            else if (randval < 0.75) p.y(p.y() + 1);
+            else                     p.y(p.y() - 1);
+          }
+          else if (connectivity == 1) {
+            // random bishop move
+            if (randval < 0.25)      {p.x(p.x() + 1); p.y(p.y() + 1);}
+            else if (randval < 0.5)  {p.x(p.x() + 1); p.y(p.y() - 1);}
+            else if (randval < 0.75) {p.x(p.x() - 1); p.y(p.y() + 1);}
+            else                     {p.x(p.x() - 1); p.y(p.y() - 1);}
+          }
+          else {
+            // random king move
+            if (randval < 0.125)      {p.y(p.y() - 1); p.x(p.x() - 1);}
+            else if (randval < 0.25)  {p.y(p.y() - 1);}
+            else if (randval < 0.375) {p.y(p.y() - 1); p.x(p.x() + 1);}
+            else if (randval < 0.5)   {p.x(p.x() + 1);}
+            else if (randval < 0.625) {p.x(p.x() + 1); p.y(p.y() + 1);}
+            else if (randval < 0.75)  {p.y(p.y() + 1);}
+            else if (randval < 0.875) {p.x(p.x() - 1); p.y(p.y() + 1);}
+            else                      {p.x(p.x() - 1);}
+          }
+          speckles->set(p,blackval);
+        }
+      }
+    }
+  }
+
+  // do a morphological closing
+  if (k>1) {
+    typename view_type::vec_iterator q;
+    // build structuring element
+    data_type* se_data = new data_type(Dim(k,k), Point(0,0));
+    view_type* se = new view_type(*se_data);
+    for (q=se->vec_begin(); q!=se->vec_end(); q++)
+      *q = blackval;
+    view_type* dilated = dilate_with_structure(*speckles, *se, Point(k/2,k/2));
+    view_type* closed = erode_with_structure(*dilated, *se, Point(k/2,k/2));
+    delete dilated->data(); delete dilated;
+    delete speckles->data(); delete speckles;
+    delete se_data; delete se;
+    speckles = closed;
+  }
+
+  // subtract speckles from input image
+  for (y=0; y <= maxy; y++) {
+    for (x=0; x <= maxx; x++) {
+      Point p(x,y);
+      if (is_black(speckles->get(p))) {
+        speckles->set(p,whiteval);
+      } else {
+        speckles->set(p,src.get(p));
+      }
+    }
+  }
+
+  return speckles;
 }
 
-template <class T>
-inline T norm_weight_avg(T& pix1, T& pix2, double w1=1.0, double w2=1.0)
-{
-  return T(((pix1 * w1) + (pix2 * w2))/(w1 + w2));
+
 }
 
- #endif
+#endif
