@@ -1,7 +1,8 @@
 /*
  *
- * Copyright (C) 2001-2005 Ichiro Fujinaga, Michael Droettboom,
- * and Karl MacMillan
+ * Copyright (C) 2001-2005 Ichiro Fujinaga, Michael Droettboom, Karl MacMillan
+ *               2009-2010 Christoph Dalitz
+ *               2010      Robert Butz
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -29,14 +30,13 @@
 #include <vector>
 
 namespace Gamera {
-  /*
-    Black Area
-
-    Find the number of black pixels in an image. This is just a convenience
-    routine, but it demonstrates the correct method for determining whether
-    a pixel is black or white.
-  */
-
+  //
+  // Black Area
+  //
+  // Find the number of black pixels in an image. This is just a convenience
+  // routine, but it demonstrates the correct method for determining whether
+  // a pixel is black or white.
+  //
   template<class T>
   void black_area(const T& mat, feature_t* buf) {
     *buf = 0;
@@ -46,9 +46,7 @@ namespace Gamera {
         (*buf)++;
     }
   }
-
   // Old-style version, since it is called from C++ elsewhere
-
   template<class T>
   feature_t black_area(const T& mat) {
     int black_pixels = 0;
@@ -61,7 +59,6 @@ namespace Gamera {
   }
 
   // Ratio of black to white pixels
-
   template<class T>
   feature_t volume(const T &m) {
     unsigned int count = 0;
@@ -78,18 +75,18 @@ namespace Gamera {
   }
   
 
-  /*
-    MOMENTS
-
-    Returns vector containing:
-    u10, u01, u20, u02, u11, u30, u12, u21, u03
-  */
+  //
+  // Normalized Central Moments 
+  //
+  // Returns vector containing:
+  // u10, u01, u20, u02, u11, u30, u12, u21, u03
+  //
 
   template<class Iterator>
-  void moments_1d(Iterator begin, Iterator end, size_t& m0, size_t& m1,
-                  size_t& m2, size_t& m3) {
+  void moments_1d(Iterator begin, Iterator end, feature_t& m0, feature_t& m1,
+                  feature_t& m2, feature_t& m3) {
     // zeroeth, first, second, third order on one axis
-    size_t tmp = 0;
+    feature_t tmp = 0;
     size_t x = 0;
     Iterator itx = begin;
     for (; itx != end; ++itx, x++) {
@@ -106,9 +103,9 @@ namespace Gamera {
   }
 
   template<class Iterator>
-  void moments_2d(Iterator begin, Iterator end, size_t& m11, size_t& m12,
-                  size_t& m21) {
-    size_t tmp = 0;
+  void moments_2d(Iterator begin, Iterator end, feature_t& m11, feature_t& m12,
+                  feature_t& m21) {
+    feature_t tmp = 0;
     size_t x = 0;
     Iterator itx = begin;
     for (; itx != end; itx++, x++) {
@@ -125,7 +122,7 @@ namespace Gamera {
 
   template<class T>
   void moments(T &m, feature_t* buf) {
-    size_t m10 = 0, m11 = 0, m20 = 0, m21 = 0, m12 = 0, 
+    feature_t m10 = 0, m11 = 0, m20 = 0, m21 = 0, m12 = 0, 
       m01 = 0, m02 = 0, m30 = 0, m03 = 0, m00 = 0, dummy = 0;
     moments_1d(m.row_begin(), m.row_end(), m00, m01, m02, m03);
     moments_1d(m.col_begin(), m.col_end(), dummy, m10, m20, m30);
@@ -174,22 +171,28 @@ namespace Gamera {
   
   template<class Iterator>
   inline int nholes_1d(Iterator begin, Iterator end) {
+
     int hole_count = 0;
     bool last;
+    bool has_black;
     Iterator r = begin;
     for (; r != end; r++) {
       last = false;
+      has_black = false;
       typename Iterator::iterator c = r.begin();
       for (; c != r.end(); c++) {
-	if (is_black(*c))
-	  last = true;
-	else if (last) {
-	  last = false;
-	  hole_count++;
-	}
+        if (is_black(*c)){
+          last = true;
+          has_black = true;
+        }
+        else if (last) {
+          last = false;
+          hole_count++;
+        }
+      }
+      if (!last && hole_count && has_black){ 
+        hole_count--;
       }
-      if (!last && hole_count)
-	hole_count--;
     }
     return hole_count;
   }
@@ -205,28 +208,28 @@ namespace Gamera {
     *buf = (feature_t)horiz / m.nrows();
   }
 
-  /*
-    nholes_extended
-
-    This divides the image into quarters (both horizontally
-    and vertically) and computes the number of holes on
-    each of the sections.
-  */
+  //
+  // nholes_extended
+  //
+  // This divides the image into strips (both horizontally
+  // and vertically) and computes the number of holes on each strip.
+  //
   template<class T>
   void nholes_extended(const T& m, feature_t* buf) {
     double quarter_cols = m.ncols() / 4.0;
     double start = 0.0;
     for (size_t i = 0; i < 4; ++i) {
       *(buf++) = nholes_1d(m.col_begin() + size_t(start),
-			   m.col_begin() + size_t(start) + size_t(quarter_cols));
+			    m.col_begin() + size_t(start) + size_t(quarter_cols))
+                /quarter_cols;
       start += quarter_cols;
     }
     double quarter_rows = m.nrows() / 4.0;
     start = 0.0;
     for (size_t i = 0; i < 4; ++i) {
       *(buf++) = nholes_1d(m.row_begin() + size_t(start),
-			   m.row_begin() + size_t(start)
-			   + size_t(quarter_rows));
+			   m.row_begin() + size_t(start) + size_t(quarter_rows))
+               / quarter_rows;
       start += quarter_rows;
     }
   }
@@ -251,12 +254,97 @@ namespace Gamera {
     *buf = feature_t(image.ncols());
   }
 
-  /*
-    compactness
-    
-    compactness is ratio of the volume of the outline of an image to
-    the volume of the image.
-  */
+
+  // helper function for compactness which computes the surface
+  // of pixels lying on the image bounding box that are ignored
+  // by a dilation
+  template<class T>
+  feature_t compactness_border_outer_volume(const T &m) {
+    int lastPx = 0;
+    int i, rows, cols;
+    int startPx = 0;
+    feature_t num_dil_px = 0;
+    rows = m.nrows();
+    cols = m.ncols();
+
+    startPx = m.get(Point(0,0)); // needed for the last pxs
+
+    // 1st row left to right, including both corners
+    for (i=0; i<cols; i++) {
+      if (is_black(m.get(Point(i,0)))) {
+        if (lastPx == 2)       num_dil_px += 1;
+        else if (lastPx == 1)  num_dil_px += 2;
+        else                   num_dil_px += 3;
+        if (i == 0 || i== rows-1) // upper left or upper right corner
+          num_dil_px += 2;
+        lastPx = 2;
+      } else { // px is white
+        lastPx -=1;
+        if (i== rows-1) // upper right corner
+          lastPx = 0;
+      }
+    }
+    // last column top-> down, including lower right corner
+    for (i=1; i<rows; i++) {
+      if (is_black( m.get(Point(cols-1,i)))) {
+        if (lastPx == 2)       num_dil_px += 1;
+        else if (lastPx == 1)  num_dil_px += 2;
+        else                   num_dil_px += 3;
+        if (i== rows-1)// lower right corner
+          num_dil_px += 2;
+        lastPx = 2;
+      } else { // px is white
+        lastPx -=1;
+        if (i== rows-1)// lower right corner
+          lastPx = 0;
+      }
+    }
+    // last row right to left, including lower left corner
+    for (i=cols-2; i>=0; i--) {
+      if (is_black(m.get(Point(i,rows-1)))) {
+        if (lastPx == 2)       num_dil_px += 1;
+        else if (lastPx == 1)  num_dil_px += 2;
+        else                   num_dil_px += 3;
+        if (i== 0)// lower left corner
+          num_dil_px += 2;
+        lastPx = 2;
+      }
+      else{ // px is white
+        lastPx -=1;
+        if (i== 0) // lower left corner
+          lastPx = 0;
+      }
+    }
+    // 1st column down->top, no corners included
+    for (i=rows-2; i>0; i--) {
+      if (is_black(m.get(Point(0,i)))) {
+        if (lastPx == 2)       num_dil_px += 1;
+        else if (lastPx == 1)  num_dil_px += 2;
+        else                   num_dil_px += 3;
+        lastPx = 2;
+      } else { // px is white
+        lastPx -=1;
+      }
+    }
+
+    // avoiding overlapping dilated_px: Start-End
+    if (is_black(startPx)) {
+      if (is_black(m.get(Point(0,1)))) {
+        num_dil_px -= 2;
+      }
+      else if (is_black( m.get(Point(0,2)))) {
+        num_dil_px -= 1;
+      }
+    }
+    return num_dil_px / (rows*cols); // volume
+  }
+
+  //
+  // compactness
+  //
+  // compactness is ratio of the volume of the outline of an image to
+  // the volume of the image.
+  //
   template<class T>
   void compactness(const T& image, feature_t* buf) {
     // I've converted this to a more efficient method.  Rather than
@@ -265,7 +353,10 @@ namespace Gamera {
     // prevents the unnecessary xor_image pixel-by-pixel operation from
     // happening.  We still need to create a copy to dilate, however,
     // since we don't want to change the original.
+    // as dilate does not extend beyond the image borders,
+    // we must compute the surface of the border pixels separately
     feature_t vol = volume(image);
+    feature_t outer_vol = compactness_border_outer_volume(image);
     feature_t result;
     if (vol == 0)
       result = std::numeric_limits<feature_t>::max();
@@ -273,147 +364,201 @@ namespace Gamera {
       typedef typename ImageFactory<T>::view_type* view_type;
       view_type copy = erode_dilate(image, 1, 0, 0);
       // dilate(*copy);
-      result = (volume(*copy) - vol) / vol;
+      result = (volume(*copy) + outer_vol - vol) / vol;
       delete copy->data();
       delete copy;
     }
     *buf = result;
   }
 
-  /*
-    volume16regions
-
-    This function divides the image into 16 regions and takes the volume of
-    each of those regions.
-  */
+  //
+  // volume16regions
+  //
+  // This function divides the image into 16 regions and takes the volume of
+  // each of those regions.
+  //
   template<class T>
   void volume16regions(const T& image, feature_t* buf) {
-    double rows = image.height() / 4.0;
-    double cols = image.width() / 4.0;
-    size_t rows_int = size_t(rows + 1);
-    size_t cols_int = size_t(cols + 1);
+    double rows = image.nrows() / 4.0;
+    double cols = image.ncols() / 4.0;
+    size_t rows_int = size_t(rows);
+    size_t cols_int = size_t(cols);
     Dim size(cols_int, rows_int);
+    if (size.ncols() == 0)
+        size.ncols(1);
+    if (size.nrows() == 0)
+        size.nrows(1);
     double start_col = double(image.offset_x());
     for (size_t i = 0; i < 4; ++i) {
       double start_row = double(image.offset_y());
       for (size_t j = 0; j < 4; ++j) {
-	T tmp(image, Point((size_t)start_col, (size_t)start_row), size);
-	*(buf++) = volume(tmp);
-	start_row += rows;
+        T tmp(image, Point((size_t)start_col, (size_t)start_row), size);
+        *(buf++) = volume(tmp);
+        start_row += rows;
+        size.nrows( size_t(start_row + rows)-size_t(start_row));
+        if (size.nrows() == 0)
+            size.nrows(1);
       }
       start_col += cols;
+      size.ncols( size_t(start_col + cols) - size_t(start_col));
+      if (size.ncols() == 0)
+        size.ncols(1);
     }
   }
 
-  /*
-    volume64regions
-
-    This function divides the image into 64 regions and takes the volume of
-    each of those regions.
-  */
+  //
+  // volume64regions
+  //
+  // This function divides the image into 64 regions and takes the volume of
+  // each of those regions.
+  //
   template<class T>
   void volume64regions(const T& image, feature_t* buf) {
-    double rows = image.height() / 8.0;
-    double cols = image.width() / 8.0;
-    size_t rows_int = size_t(rows + 1);
-    size_t cols_int = size_t(cols + 1);
+    double rows = image.nrows() / 8.0;
+    double cols = image.ncols() / 8.0;
+    size_t rows_int = size_t(rows);
+    size_t cols_int = size_t(cols);
     Dim size(cols_int, rows_int);
+    if (size.ncols() == 0)
+        size.ncols(1);
+    if (size.nrows() == 0)
+        size.nrows(1);
     double start_col = double(image.offset_x());
     for (size_t i = 0; i < 8; ++i) {
       double start_row = double(image.offset_y());
       for (size_t j = 0; j < 8; ++j) {
-	T tmp(image, Point((size_t)start_col, (size_t)start_row), size);
-	*(buf++) = volume(tmp);
-	start_row += rows;
+        T tmp(image, Point((size_t)start_col, (size_t)start_row), size);
+        *(buf++) = volume(tmp);
+        start_row += rows;
+        size.nrows( size_t(start_row + rows)-size_t(start_row));
+        if (size.nrows() == 0)
+            size.nrows(1);
       }
       start_col += cols;
+      size.ncols( size_t(start_col + cols) - size_t(start_col));
+      if (size.ncols() == 0)
+            size.ncols(1);
     }
   }
 
-  double zer_pol_R(int n, int m_in, double x, double y) {
-    int m = abs(m_in);
-    int sign;
+
+  //
+  // Zernike Moments
+  //
+
+  double zer_pol_R(int n, int m, double x, double y) {
+    // precomputed factorials => make sure that n < 11
+    const int fak_a[] = {1,1,2,6,24,120,720,5040,40320,362880,3628800};
+    int s,Na,Nb,Nc;
+    int sign = 1;
     double result = 0;
-    double distance = (x * x + y * y);
-      sign = 1;
-    int a = 1;
-    for (int i = 2; i <= n; ++i)
-      a *= i;
-    int b = 1;
-    int c = 1;
-    for (int i = 2; i <= (n + m) / 2; ++i)
-      c *= i;
-    int d = c;
-    int s = 0; // Outside the loop, since we need to access it at the end too.
-    for (; s < (n - m) / 2; ++s) {
-      result += sign * (a * 1.0 / (b * c * d)) * pow(distance, (n / 2.0) - s);
+    double distance = sqrt(x * x + y * y);
+    double Zb = 1;
+
+    for(s=0; s<=(n-m)/2; s++){
+      Na = fak_a[n-s] / fak_a[s];
+      if (n-2*s == 0)
+        Zb = 1;
+      else
+        Zb *= distance;
+      Nb = fak_a[(n+m)/2-s];
+      Nc = fak_a[(n-m)/2-s];
+      result += sign * (Na * Zb / Nb) / Nc;
       sign = -sign;
-      a /= (n - s);
-      b *= (s + 1);
-      c /= ((n + m) / 2 - s);
-      d /= ((n - m) / 2 - s);
     }
-    result += sign * (a * 1.0 / (b * c * d)) * pow(distance, (n / 2.0) - s);
     return result;
   }
 
-  void zer_pol(int n, int m, double x, double y, double& real, double& imag) {
-    if ((x*x + y*y) > 1.0) {
+  void zer_pol(int n, int m, double x, double y, double& real, double& imag, double norm_scale=1.0) {
+    const complex<double> I(0.0, 1.0);
+    // theoretically redundant due to scaling,
+    // but with translation-normalizing after all needed
+    if (sqrt(x*x + y*y) > 1.0) {
+      //std::cout << "outer px=" << x << "," << y << std::endl;
       real = 0.0;
       imag = 0.0; 
     } else {
-      double R = zer_pol_R(n, m, x, y);
-      double arg = m * atan2(y, x);
-      real = R * cos(arg);
-      imag = R * sin(arg);
+      double Rnm = zer_pol_R(n, m, x*norm_scale, y*norm_scale);
+      double angle_Theta = atan2(y,x);
+      complex<double> Inm = exp(m*angle_Theta*I);
+      complex<double> result = conj(Rnm * Inm); // complex conj.
+      real = result.real();
+      imag = result.imag();
     }
   }
 
-  // This is my own modification so that all pixels in the image become 
-  // involved in the calculation.  I "imagine" a zernike circle that encompasses the
-  // entire bounding box rectangle.  (Rather than an ellipse that fits
-  // inside the bounding box.)  This not only helps to better distinguish
-  // glyphs, it is slightly more efficient since it removes an 'if' in the
-  // inner loop.
+  // we use this wrapper so that it is easy to
+  // change the maximum order in the future
   template<class T>
   void zernike_moments(const T& image, feature_t* buf) {
-    size_t max_dimension = std::max(image.ncols(), image.nrows());
-    double x_0 = (image.ncols() + 1) / 2.0;
-    double y_0 = (image.nrows() + 1) / 2.0;
-    double scale = max_dimension / 2.0;
+    // beware that, when changing the maximum order from six to
+    // something different, the dimension of the feature vector
+    // needs to be adjusted in the python interface features.py
+    zernike_moments( image, buf, 6);
+  }
+
+  template<class T>
+  void zernike_moments(const T& image, feature_t* buf, size_t order_n) {
+    size_t const max_order_n=order_n; 
+    size_t num_features=0; // evaluated by max_order_n
     double x_dist, y_dist, real_tmp, imag_tmp;
 
-    int m = 1;
+    // compute center of mass and normalization factor m00
+    feature_t m00=0, m10=0, m01=0, dummy1=0, dummy2=0, dummy3=0;
+    moments_1d(image.row_begin(), image.row_end(), m00, m01, dummy1, dummy2);
+    moments_1d(image.col_begin(), image.col_end(), dummy1, m10, dummy2, dummy3);
+    double centroid_x = m10/m00;
+    double centroid_y = m01/m00;
+
+    // we use a Zernike circle that includes the entire image
+    // beware however that some pixels can fall outside the circle
+    // by normalizing ZMs to be translation invariant, e.g. a large
+    // bunch of pixels in the upper left corner which draws the
+    // center to it, excludes pixels in the lower right corner.
+    size_t max_dimension = std::max(image.ncols(), image.nrows());
+    double unit_circle_scale = sqrt(2*pow(max_dimension/2.0,2));
+
+    // number of features depends on maximum order
+    for (size_t i=0 ; i<=max_order_n; i++)
+      num_features += i/2 + 1;
+    num_features -= 2; // A00 and A11 are constants
 
     feature_t* begin = buf;
-    for (size_t i = 0; i < 26; ++i)
+    for (size_t i = 0; i < num_features; ++i)
       *(buf++) = 0.0;
     buf = begin;
 
+    size_t m, n, idx;
     typename T::const_vec_iterator it = image.vec_begin();
-    for (size_t y = 0; y < image.nrows(); ++y)
+    for (size_t y = 0; y < image.nrows(); ++y) {
       for (size_t x = 0; x < image.ncols(); ++x, ++it) {
-	y_dist = (y - y_0) / scale;
-	x_dist = (x - x_0) / scale;
-	if (is_black(*it)) {
-	  for (size_t n = 1; n < 14; ++n) { 
-	    size_t idx = (n - 1) * 2;
-	    zer_pol(n, m, x_dist, y_dist, real_tmp, imag_tmp);
-	    buf[idx] += real_tmp;
-	    buf[idx + 1] += (-imag_tmp);
-	  }
-	}
+        if (is_black(*it)) {
+          x_dist = (x - centroid_x) / unit_circle_scale;
+          y_dist = (y - centroid_y) / unit_circle_scale;
+          for (n = 2, idx=0; n <= max_order_n; ++n) {
+            for (m = n%2; m <= n; m+=2) {
+              zer_pol(n, m, x_dist, y_dist, real_tmp, imag_tmp);
+              buf[idx++] += sqrt(real_tmp*real_tmp + imag_tmp*imag_tmp);
+            }
+          }
+        }
       }
-    
-    for (size_t n = 1; n < 14; ++n) {
-      size_t idx = (n-1) * 2;
+    }
+
+    // scale normalization by m00
+    for (size_t n = 2, idx=0; n <= max_order_n; ++n) {
       double multiplier = (n + 1) / M_PI;
-      buf[idx] *= multiplier;
-      buf[idx + 1] *= multiplier;
+      multiplier /= m00;
+      for (m= n%2; m<= n; m+=2){
+        buf[idx++] *= multiplier;
+      }
     }
 
   }
 
+  //
+  // Skeleton features
+  //
   template<class T>
   void skeleton_features(const T& image, feature_t* buf) {
     if (image.nrows() == 1 || image.ncols() == 1) {
@@ -474,20 +619,23 @@ namespace Gamera {
     bool last_pixel = false;
     for (size_t y = 0; y < skel->nrows(); ++y)
       if (is_black(skel->get(Point(center_x, y))) && !last_pixel) {
-	last_pixel = true;
-	++x_axis_crossings;
-      } else 
-	last_pixel = false;
+        last_pixel = true;
+        ++x_axis_crossings;
+      } else {
+        last_pixel = false;
+      }
   
     center_y /= total_pixels;
     size_t y_axis_crossings = 0;
     last_pixel = false;
-    for (size_t x = 0; x < skel->ncols(); ++x)
+    for (size_t x = 0; x < skel->ncols(); ++x) {
       if (is_black(skel->get(Point(x, center_y))) && !last_pixel) {
-	last_pixel = true;
-	++y_axis_crossings;
-      } else 
-	last_pixel = false;
+        last_pixel = true;
+        ++y_axis_crossings;
+      } else {
+        last_pixel = false;
+      }
+    }
     
     delete skel->data();
     delete skel;
@@ -500,6 +648,9 @@ namespace Gamera {
     *buf = feature_t(y_axis_crossings);
   }
 
+  //
+  // Top Bottom
+  //
   template<class T>
   void top_bottom(const T& m, feature_t* buf) {
     int top = -1;
