@@ -1,7 +1,7 @@
 /*
  *
- * Copyright (C) 2001-2005
- * Ichiro Fujinaga, Michael Droettboom, and Karl MacMillan
+ * Copyright (C) 2001-2005 Ichiro Fujinaga, Michael Droettboom, Karl MacMillan
+ *               2010      Christoph Dalitz, Hasan Yildiz, Tobias Bolten
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -30,7 +30,11 @@
 #include <exception>
 #include <math.h>
 #include <algorithm>
+#include <map>
 
+// for compatibility: resize, scale, mirror, and shear
+//  were formerly implemented in image_utilitis instead of transformation
+#include "transformation.hpp"
 
 namespace Gamera {
   
@@ -247,51 +251,6 @@ namespace Gamera {
     return dest;
   }
 
-  template<class T>
-  Image* resize(T& image, const Dim& dim, int resize_quality) {
-    typename T::data_type* data = new typename T::data_type
-      (dim, image.origin());
-    ImageView<typename T::data_type>* view = 
-      new ImageView<typename T::data_type>(*data);
-    /*
-      Images with nrows or ncols == 1 cannot be scaled by VIGRA.
-      This is a hack that just returns an image with the same 
-      color as the upper-left pixel
-    */
-    if (image.nrows() <= 1 || image.ncols() <= 1 || 
-	view->nrows() <= 1 || view->ncols() <= 1) {
-      std::fill(view->vec_begin(), view->vec_end(), image.get(Point(0, 0)));
-      return view;
-    }
-    if (resize_quality == 0) {
-      // for straight scaling, resampleImage must be used in VIGRA
-      double xfactor = (double)view->ncols()/image.ncols();
-      double yfactor = (double)view->nrows()/image.nrows();
-      // this is implemented incorrectly in VIGRA:
-      //resizeImageNoInterpolation(src_image_range(image), dest_image_range(*view));
-      // the following works however:
-      // requires extension of VIGRA (see basicgeometry.hxx)
-      // that are not yet merged into VIGRA 1.6.0
-      resampleImage(src_image_range(image), dest_image(*view), xfactor, yfactor);
-    } else if (resize_quality == 1) {
-      resizeImageLinearInterpolation(src_image_range(image), dest_image_range(*view));
-    } else {
-      resizeImageSplineInterpolation(src_image_range(image), dest_image_range(*view));
-    }
-    image_copy_attributes(image, *view);
-    return view;
-  }
-
-  template<class T>
-  Image* scale(T& image, double scaling, int resize_quality) {
-    // nrows, ncols are cast to a double so that the multiplication happens
-    // exactly as it does in Python
-    return resize(image, 
-		  Dim(size_t(double(image.ncols()) * scaling),
-		      size_t(double(image.nrows()) * scaling)),
-		  resize_quality);
-  }
-
 
   /*
     FloatVector histogram(GreyScale|Grey16 image);
@@ -477,6 +436,62 @@ namespace Gamera {
   }
 
 
+  /*
+    Trim white (or other) borders
+  */
+  template<class T>
+  Image * trim_image(const T &image, const typename T::value_type pixelValue) {
+    typedef typename T::value_type value_type;
+    typedef typename ImageFactory<T>::view_type view_type;
+    view_type *res;
+
+    unsigned int min_x, max_x;
+    unsigned int min_y, max_y;
+
+    min_x = image.ncols() - 1;
+    min_y = image.nrows() - 1;
+
+    max_x = max_y = 0;
+
+    // Search upper left and lower right coordinates for the bounding box
+    // of the view
+    for(size_t y = 0; y < image.nrows(); ++y) {
+      for(size_t x = 0; x < image.ncols(); ++x) {
+        if( image.get(Point(x,y)) != pixelValue ) {
+          if( x < min_x )
+            min_x = x;
+          if( x > max_x )
+            max_x = x;
+          if( y < min_y )
+            min_y = y;
+          if( y > max_y )
+            max_y = y;                                
+        }
+      }
+    }
+
+    // When no points found, set the view to the complete image
+    if( min_x > max_x ) {
+      min_x = 0;
+      max_x = image.ncols() - 1;
+    }
+    if( min_y > max_y ) {
+      min_y = 0;
+      max_y = image.nrows() - 1;
+    }
+
+    // Creates points for the new view
+    // Don't forget the offset of the image (this can be already a view)
+    // --> Offsets have to be added
+    Point ul(min_x + image.offset_x(), min_y + image.offset_y());
+    Point lr(max_x + image.offset_x(), max_y + image.offset_y());
+
+    // Create the view and return it
+    res = new view_type(*image.data(), ul, lr);
+    return res;
+  }
+
+
   template<class T>
   void invert(T& image) {
     ImageAccessor<typename T::value_type> acc;
@@ -485,48 +500,6 @@ namespace Gamera {
       acc.set(invert(acc(in)), in);
   }
 
-  /*
-    Shearing
-  */
-
-  template<class T>
-  inline void simple_shear(T begin, const T end, int distance) {
-    // short-circuit
-    if (distance == 0)
-      return;
-    typename T::value_type filler;
-    // move down or right
-    if (distance > 0) {
-      filler = *begin;
-      std::copy_backward(begin, end - distance, end);
-      std::fill(begin, begin + distance, filler);
-      // move up or left
-    } else if (distance < 0) {
-      filler = *(end - 1);
-      std::copy(begin - distance, end, begin);
-      std::fill(end + distance, end, filler);
-    } // if distance == 0, do nothing
-  }
-
-  template<class T>
-  void shear_column(T& mat, size_t column, int distance) {
-    if (size_t(std::abs(distance)) >= mat.nrows())
-      throw std::range_error("Tried to shear column too far");
-    if (column >= mat.ncols())
-      throw std::range_error("Column argument to shear_column out of range");
-    simple_shear((mat.col_begin() + column).begin(),
-		 (mat.col_begin() + column).end(), distance);
-  }
-
-  template<class T>
-  void shear_row(T& mat, size_t row, int distance) {
-    if (size_t(std::abs(distance)) >= mat.ncols())
-      throw std::range_error("Tried to shear column too far");
-    if (row >= mat.nrows())
-      throw std::range_error("Column argument to shear_column out of range");
-    simple_shear((mat.row_begin() + row).begin(),
-		 (mat.row_begin() + row).end(), distance);
-  }
 
   template<class T>
   Image *clip_image(T& m, const Rect* rect) {
@@ -717,27 +690,6 @@ namespace Gamera {
     return rows;
   }
 
-  template<class T>
-  void mirror_horizontal(T& m) {
-    for (size_t r = 0; r < size_t(m.nrows()) / 2; ++r) {
-      for (size_t c = 0; c < m.ncols(); ++c) {
-	typename T::value_type tmp = m.get(Point(c, r));
-	m.set(Point(c, r), m.get(Point(c, m.nrows() - r - 1)));
-	m.set(Point(c, m.nrows() - r - 1), tmp);
-      }
-    }
-  }
-
-  template<class T>
-  void mirror_vertical(T& m) {
-    for (size_t r = 0; r < m.nrows(); ++r) {
-      for (size_t c = 0; c < size_t(m.ncols() / 2); ++c) {
-	typename T::value_type tmp = m.get(Point(c, r));
-	m.set(Point(c, r), m.get(Point(m.ncols() - c - 1, r)));
-	m.set(Point(m.ncols() - c - 1, r), tmp);
-      }
-    }
-  }
 
   template<class T>
   double mse(T& a, T& b) {
@@ -763,5 +715,61 @@ namespace Gamera {
     }
   }
 
+  /*
+   * compute Cc's from an already labeled image
+   * Christoph Dalitz and Hasan Yildiz
+   */
+  template<class T>
+  ImageList* ccs_from_labeled_image(T &src) {
+    typedef typename T::value_type value_type;
+    value_type value;
+
+    ImageList* return_ccs = new ImageList();
+
+    std::map<unsigned int, Rect*> pixel;
+    std::map<unsigned int, Rect*>::iterator iter;
+
+    for (size_t y=0; y < src.nrows(); ++y) {
+      for (size_t x=0; x < src.ncols(); ++x) {
+        if (!is_white(src.get(Point(x,y)))) {
+          value = src.get(Point(x,y));
+
+          // when new label: create Rect for new Cc
+          if (pixel.find(value) == pixel.end()) {
+            pixel[value] = new Rect(Point(x, y), Point(x, y));
+          }
+          // update Rect bounding box when known label
+          else {
+            iter = pixel.find(value);
+
+            if (y < (*iter).second->ul_y())
+              (*iter).second->ul_y(y);
+            if (x < (*iter).second->ul_x())
+              (*iter).second->ul_x(x);
+            if (y > (*iter).second->lr_y())
+              (*iter).second->lr_y(y);
+            if (x > (*iter).second->lr_x())
+              (*iter).second->lr_x(x);
+          }
+        }
+      }
+    }
+
+    // create Cc's for all labels
+    for (iter = pixel.begin(); iter != pixel.end(); iter++) {
+      return_ccs->push_back(new ConnectedComponent<typename T::data_type>(
+                    *src.data(),    // data
+                    (*iter).first,  // label
+                    Point((*iter).second->ul_x(), (*iter).second->ul_y()), // upper left
+                    Point((*iter).second->lr_x(), (*iter).second->lr_y())  // lower right
+                  ));
+
+      delete iter->second;
+      iter->second = 0;
+    }
+
+    return return_ccs;
+  }
+
 }
 #endif
